;nyquist plug-in
;version 3
;type process
;categories "http://lv2plug.in/ns/lv2core/#MixerPlugin"
;name "Mid-Side..."
;action "Converting..."
;info "by Steve Daulton (http://easyspacepro.com). Released under GPL v2."

;; MidSide.ny by Steve Daulton, Jan 2012
;; Released under terms of the GNU General Public License version 2
;; http://www.gnu.org/copyleft/gpl.html

;control mode "M/S decode, encode or view help" choice "Decode,Encode,View Help" 0 
;control chan "Stereo channel containing Mid" choice "Left (upper),Right (lower)" 0 
;control width "Side Channel Gain (dB).  Narrow<-\n(Decode only)" real "->Wide\n" 0 -15 15
;control gain "Output Gain (dB)\n(Decode only)" real "" 0 -15 15

(defun MSmix (s-in sidegain gain &optional (encode 0) (mid 0))
  (setq side (if (= mid 0) 1 0))  ;side channel
  (setq sidegain (db-to-linear sidegain))
  (setq gain (db-to-linear gain))
  (if (= encode 0)                ;decode
    (mult gain
      (vector 
        (sum (aref s-in mid)(mult sidegain (aref s-in side)))
        (sum (aref s-in mid)(mult -1 sidegain (aref s-in side)))))
    (progn                        ;encode
      (case mid
        (0 (setq ch0 1 ch1 -1))
        (T (setq ch0 -1 ch1 1)))
      (mult 0.5
        (vector 
          (sum (aref s-in 0)(mult ch0 (aref s-in 1)))
          (sum (aref s-in 0)(mult ch1 (aref s-in 1))))))))

(defun help ()
  (setq help
"Middle-Side (M/S) stereo may be recorded using a 
cardioid mic facing the sound source to record a 
'Middle' channel and a figure 8 pattern microphone
as close as possible and at 90 degressd to the Mid
mic to record a 'Side' channel. If M/S mics are 
recorded as a stereo pair without first decoding,
it is customary for the Mid mic to be recorded in 
the first (left) channel and the Side mic recorded
to the second channel.

Decoding M/S involves playing the Mid channel
through both speakers, adding the Side channel to
the left speaker and the inverted Side channel to 
the right speaker:

The 'Output Gain' control amplifies the processed
signal to a higher or lower level and can help
compensate for increased level when using wide 
settings.

The plug-in can also create M/S encoded recordings
from a standard Left/Right stereo recording by
selecting the 'Encode' option. Selecting which
channel to use as the Mid channel is the only
user setting for the M/S encoder.")
  (format t "~a" help)    ;print to debug window
  (format nil "~a" help)) ;output help screen

(cond
  ((= mode 2) (help))           ;View Help
  ((not (arrayp s))             ;Not stereo track
    (format nil "Error.\n2 channel (stereo) track required."))
  ((and (= mode 0)(= chan 0))   ;Decode. Mid left
    (MSmix s width gain))
  ((= mode 0)                   ;Decode. Mid Right
    (MSmix s width gain 0 1))
  ((= chan 0)                   ;Encode. Mid Left
    (MSmix s width gain 1))
  (T                            ;Encode. Mid Right
    (MSmix s width gain 1 1)))